//-----------------------------------------------------------------------
// File : AsModelUtil.h
// Desc : Asura Model Utility
// Copyright(c) Project Asura All right reserved.
//-----------------------------------------------------------------------

//-----------------------------------------------------------------------
// History
//-----------------------------------------------------------------------
// <---> 2011/03/25 [Pocol] VK쐬
// <End>

#ifndef __ASURA_MODEL_UTIL_H__
#define __ASURA_MODEL_UTIL_H__

//-----------------------------------------------------------------------
// Includes
//------------------------------------------------------------------------
#include <GL/glew.h>
#include <vector>
#include "AsDef.h"
#include "AsModel.h"
#include "AsTimer.h"

namespace Schaft {
namespace Asura {
namespace Util {

/////////////////////////////////////////////////////////////////////////
// VERTEX_DATA_TYPE
/////////////////////////////////////////////////////////////////////////
enum VERTEX_DATA_TYPE
{
    VERTEX_DATA_POSITION = 0,
    VERTEX_DATA_NORMAL,
    VERTEX_DATA_COLOR,
    VERTEX_DATA_TEXCOORD,
    VERTEX_DATA_TANGENT,
    VERTEX_DATA_BINORMAL,
    VERTEX_DATA_SKIN_INDEX,
    VERTEX_DATA_SKIN_WEIGHT,
    NUM_VERTEX_DATA_TYPE,
};

//////////////////////////////////////////////////////////////////////////
// SubsetData structure
//////////////////////////////////////////////////////////////////////////
typedef struct _SubsetData
{
    uint32_t mIB;
    uint32_t mCount;
    uint32_t mMatIdx;
} SubsetData;

/////////////////////////////////////////////////////////////////////////
// MeshData structure
/////////////////////////////////////////////////////////////////////////
typedef struct _MeshData
{
    uint32_t mVB[NUM_VERTEX_DATA_TYPE];
    std::vector< SubsetData > mSubsets;
} MeshData;


/////////////////////////////////////////////////////////////////////////
// ModelData class
/////////////////////////////////////////////////////////////////////////
class ModelData
{
    //-------------------------------------------------------------------
    // list of friend classes and methods
    //-------------------------------------------------------------------
    /* NOTHING */

public:
    //-------------------------------------------------------------------
    // public variables
    //-------------------------------------------------------------------
    /* NOTHING */

    //-------------------------------------------------------------------
    // public methods
    //-------------------------------------------------------------------
    ModelData();
    virtual ~ModelData();
    virtual bool LoadFromX( const char *filename );
    virtual void Release();
    void         Draw();

protected:
    //-------------------------------------------------------------------
    // protected variables
    //-------------------------------------------------------------------
    std::vector< MeshData >                 mMeshes;
    Schaft::Asura::Graphics::IModel*        mpInterface;
    Schaft::Asura::Graphics::IModelLoader*  mpLoader;

    //-------------------------------------------------------------------
    // protected methods
    //-------------------------------------------------------------------
    void DrawMesh( const MeshData &mesh, const std::vector< Schaft::Asura::Graphics::IMaterial* > &materials );
    bool CreateLoader();
    bool CreateBuffer( MeshData &data, Schaft::Asura::Graphics::IMesh* pMesh );
    void ReleaseLoader();
    void ReleaseBuffer( MeshData &data );
    void ReleaseInterface();

private:
    //-------------------------------------------------------------------
    // private variables
    //-------------------------------------------------------------------
    /* NOTHING */

    //-------------------------------------------------------------------
    // private methods
    //-------------------------------------------------------------------
    /* NOTHING */
};


/////////////////////////////////////////////////////////////////////////
// SkinningModelData class
/////////////////////////////////////////////////////////////////////////
class SkinningModelData : public ModelData
{
    //-------------------------------------------------------------------
    // list of friend classes and methods
    //-------------------------------------------------------------------
    /* NOTHING */

public:
    //-------------------------------------------------------------------
    // public variables
    //-------------------------------------------------------------------
    /* NOTHING */

    //-------------------------------------------------------------------
    // public methods
    //-------------------------------------------------------------------
    SkinningModelData();
    virtual ~SkinningModelData();
    virtual bool LoadFromX( const char *filename );
    virtual void Release();
    unsigned int GetNumClips();
    void         SetClip( const unsigned int index );
    void         SetRootMatrix( const Schaft::Asura::Math::Matrix &matrix );
    void         Update( const bool isStop = false, const double speed = 1.0 );
    std::vector< Schaft::Asura::Math::Matrix >  GetSkinTransformMatrix();
    Schaft::Asura::Math::Matrix                 GetRootMatrix();

protected:
    //--------------------------------------------------------------------
    // protected variables
    //--------------------------------------------------------------------
    /* NOTHING */

    //--------------------------------------------------------------------
    // protected methods
    //--------------------------------------------------------------------
    void ReleasePlayer();

private:
    //--------------------------------------------------------------------
    // private variables
    //--------------------------------------------------------------------
    Schaft::Asura::Math::Matrix                 mRoot;
    Schaft::Asura::Graphics::IAnimationPlayer*  mpPlayer;
    Schaft::Asura::Timer                        mTimer;

    //---------------------------------------------------------------------
    // private methods
    //---------------------------------------------------------------------
    /* NOTHING */
};


} // namespace Util
} // namespace Asura
} // namespace Schaft

#endif//__ASURA_MODEL_UTIL_H__