//----------------------------------------------------------------------
// File : AsTextureUtil.cpp
// Desc : Asura Texture Util
// Copyright(c) Project Asura All right reserved.
//----------------------------------------------------------------------

//----------------------------------------------------------------------
// History
//----------------------------------------------------------------------
// <---> 2011/04/06 [Pocol] VK쐬
// <End>


//----------------------------------------------------------------------
// Includes
//----------------------------------------------------------------------
#include <cassert>
#include "AsTextureUtil.h"


//----------------------------------------------------------------------
// Using Statements
//----------------------------------------------------------------------
using namespace Schaft::Asura;
using namespace Schaft::Asura::Graphics;
using namespace Schaft::Asura::Math;

namespace Schaft {
namespace Asura {
namespace Util {

///////////////////////////////////////////////////////////////////////
// TextureData class
///////////////////////////////////////////////////////////////////////

///--------------------------------------------------------------------
///<summary>
///RXgN^
///</summary>
///--------------------------------------------------------------------
TextureData::TextureData()
{
    mpInterface = null;
    mpLoader = null;
}

///--------------------------------------------------------------------
///<summary>
///fXgN^
///</summary>
///--------------------------------------------------------------------
TextureData::~TextureData()
{
    Release();
}

///--------------------------------------------------------------------
///<summary>
///
///</summary>
///--------------------------------------------------------------------
void TextureData::Release()
{
    if ( mpLoader )
    {
        mpLoader->Release();
        delete mpLoader;
        mpLoader = null;
    }

    if ( mpInterface )
    {
        mpInterface->Release();
        mpInterface = null;
    }
}

///--------------------------------------------------------------------
///<summary>
///DDSt@CeNX`[h
///</summary>
///<param name="filename">[heNX`t@C</param>
///<returns>sꍇ̂0ԋp</returns>
///--------------------------------------------------------------------
uint32_t TextureData::LoadFromDDS( const char *filename )
{
    // [_[̐
    mpLoader = TextureLoaderFactory::Create( TextureLoaderFactory::TEXTURE_TYPE_DDS );
    if ( mpLoader == null )
    {
        ELOG( "Error : TextureLoaderFactory::Create() Failed." );
        return 0;
    }
    else
    {
        for( uint32_t i=0; i<UINT_MAX; i++ )
        {
            // [h
            if ( !mpLoader->IsLoading() )
            {
                // ͏
                if ( mpLoader->Parse() )
                {
                    // ͂ɐC^tF[X擾
                    mpInterface = mpLoader->GetTexture();

                    // GL̃eNX`𐶐ĕԋp
                    return SetupTextureDDS();
                }
            }
        }
    }

    // s
    return 0;
}

///--------------------------------------------------------------------
///<summary>
///DDSeNX`̐ݒs
///</summary>
///<returns>
///GLeNX`̔ԍԋp(1ȏオԋp)
///sꍇ0ԋp
///</returns>
///--------------------------------------------------------------------
uint32_t TextureData::SetupTextureDDS()
{
    uint32_t texID = 0;

    if ( mpInterface != null && mpLoader != null )
	{
		//@eNX`
		glGenTextures( 1, &texID );
		glBindTexture( GL_TEXTURE_2D, texID );

		//@eNX`p[^̐ݒ
		glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
		glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR );

		int blockSize = 0;
		GLenum format = 0;

		//@ubNTCYƃtH[}bg𔻒
		switch( mpInterface->GetFormat() )
		{
		case ITexture::TEXTURE_FORMAT_DXT1:
			blockSize = 8;
			format = GL_COMPRESSED_RGBA_S3TC_DXT1_EXT;
			break;

		case ITexture::TEXTURE_FORMAT_DXT3:
			blockSize = 16;
			format = GL_COMPRESSED_RGBA_S3TC_DXT3_EXT;
			break;

		case ITexture::TEXTURE_FORMAT_DXT5:
			blockSize = 16;
			format = GL_COMPRESSED_RGBA_S3TC_DXT5_EXT;
			break;

		default:
			assert( false );
			break;
		}

		GLsizei w = mpInterface->GetWidth();
		GLsizei h = mpInterface->GetHeight();
		GLsizei size = 0;
		uint32_t offset = 0;

		for( uint32_t i=0; i<mpInterface->GetNumMipmaps(); ++i )
		{
			size = ( ( w + 3 ) / 4 ) * ( ( h + 3 ) / 4 ) * blockSize;
			glCompressedTexImage2D( GL_TEXTURE_2D, i, format, w, h, 0, size, &mpInterface->GetPixels()[offset] );

			w = Max( ( w >> 1 ), 1 );
			h = Max( ( h >> 1 ), 1 );

			offset += size;
		}

		//@eNX`AoCh
		glBindTexture( GL_TEXTURE_2D, 0 );

		//@[_[̃
		mpLoader->Release();
        delete mpLoader;
        mpLoader = null;
	}
    return texID;
}

uint32_t TextureData::GetWidth()
{
    if ( mpInterface )
    {
        return mpInterface->GetWidth();
    }
    return 0;
}

uint32_t TextureData::GetHeight()
{
    if ( mpInterface )
    {
        return mpInterface->GetHeight();
    }
    return 0;
}

uint16_t TextureData::GetBitsPerPixel()
{
    if ( mpInterface )
    {
        return mpInterface->GetBitsPerPixel();
    }
    return 0;
}

uint32_t TextureData::GetTextureFormat()
{
    if ( mpInterface )
    {
        return mpInterface->GetFormat();
    }
    return ITexture::TEXTURE_FORMAT_NONE;
}

} // namespace Util
} // namespace Asura
} // namespace Schaft