//----------------------------------------------------------------------
// File : AsTextureImpl.cpp
// Desc : Asura Texture Implement
// Copyright(c) 2010 Project Asura All right reserved.
//----------------------------------------------------------------------

//----------------------------------------------------------------------
// History
//----------------------------------------------------------------------
// <---> 2010/11/28 [Pocol] VK쐬.
// <End>

//----------------------------------------------------------------------
// Includes
//----------------------------------------------------------------------
#include "AsTextureImpl.h"
#include "AsTextureLoaderDDS.h"
#include <cassert>

namespace Schaft {
namespace Asura {
namespace Graphics {

/////////////////////////////////////////////////////////////////////////
// Texture class
/////////////////////////////////////////////////////////////////////////

///----------------------------------------------------------------------
///<summary>
///RXgN^
///</summary>
///----------------------------------------------------------------------
Texture::Texture()
{
	mWidth = 0;
	mHeight = 0;
	mFormat = 0;
	mBitsPerPixel = 0;
	mSize = 0;
	mpBuffers = null;
	mNumMipmaps = 0;
	mName.clear();
};

///----------------------------------------------------------------------
///<summary>
///fXgN^
///</summary>
///----------------------------------------------------------------------
Texture::~Texture()
{
	Release();
}

///----------------------------------------------------------------------
///<summary>
///
///</summary>
///----------------------------------------------------------------------
void Texture::Release()
{
	mWidth = 0;
	mHeight = 0;
	mFormat = 0;
	mBitsPerPixel = 0;
	mSize = 0;
	mNumMipmaps = 0;
	SAFE_DELETE_ARRAY( mpBuffers );
	mName.clear();
}

///----------------------------------------------------------------------
///<summary>
///obt@̃TCY
///</summary>
///<param name="size">mۂ郁TCY</param>
///----------------------------------------------------------------------
void Texture::Resize( uint32_t size )
{
	SAFE_DELETE_ARRAY( mpBuffers );
	try 
	{
		mSize = size;
		mpBuffers = new char [size];
	}
	catch( std::bad_alloc ba )
	{
		ELOG( "Error : Memory Allocate Failed -%s", ba.what() );
		return;
	}
}

///----------------------------------------------------------------------
///<summary>
///obt@|C^ݒ肷
///</summary>
///----------------------------------------------------------------------
void Texture::SetPixels( char *pBuffer )
{
	memcpy( mpBuffers, pBuffer, mSize );
}

///----------------------------------------------------------------------
///<summary>
///w肳ꂽCfbNXɒlݒ肷
///</sumamry>
///<param name="index">li[obt@̃CfbNX</param>
///<param name="value">i[l</param>
///----------------------------------------------------------------------
void Texture::SetPixel( uint32_t index, char value )
{
	assert( index < mSize );
	assert( mpBuffers != 0 );
	mpBuffers[index] = value;
}

///----------------------------------------------------------------------
///<summary>
///摜ݒ肷
///</summary>
///<param name="width">ݒ肷摜</param>
///----------------------------------------------------------------------
void Texture::SetWidth( uint32_t width )
{
	mWidth = width;
}

///----------------------------------------------------------------------
///<summary>
///摜ݒ肷
///</summary>
///<param name="height">ݒ肷摜</param>
///----------------------------------------------------------------------
void Texture::SetHeight( uint32_t height )
{
	mHeight = height;
}

///----------------------------------------------------------------------
///<summary>
///1sNZ̃rbgݒ肷
///</summary>
///<param name="bpp">ݒ肷1sNZ̃rbg</param>
///----------------------------------------------------------------------
void Texture::SetBitsPerPixel( uint16_t bpp )
{
	mBitsPerPixel = bpp;
}

///----------------------------------------------------------------------
///<summary>
///tH[}bgݒ肷
///</summary>
///<param name="format">ݒ肷tH[}bg</param>
///----------------------------------------------------------------------
void Texture::SetFormat( uint32_t format )
{
	mFormat = format;
}

///----------------------------------------------------------------------
///<summary>
///t@Cݒ肷
///</summary>
///<param name="name">ݒ肷t@C</param>
///----------------------------------------------------------------------
void Texture::SetName( const std::string &name )
{
	mName = name;
}

///----------------------------------------------------------------------
///<summary>
///~bv}bvݒ肷
///</summary>
///<param name="num_mipmaps">ݒ肷~bv}bv</param>
///----------------------------------------------------------------------
void Texture::SetNumMipmaps( uint32_t num_mipmaps )
{
	mNumMipmaps = num_mipmaps;
}

///----------------------------------------------------------------------
///<summary>
///摜擾
///</summary>
///<returns>摜̕ԋp</returns>
///----------------------------------------------------------------------
uint32_t Texture::GetWidth()
{
	return mWidth; 
}

///----------------------------------------------------------------------
///<summary>
///摜擾
///</summary>
///<returns>摜̍ԋp</returns>
///----------------------------------------------------------------------
uint32_t Texture::GetHeight()
{
	return mHeight;
}

///----------------------------------------------------------------------
///<summary>
///1sNZ̃rbg擾
///</summary>
///<returns>1sNZl̃rbgԋp</returns>
///----------------------------------------------------------------------
uint16_t Texture::GetBitsPerPixel()
{
	return mBitsPerPixel;
}

///----------------------------------------------------------------------
///<summary>
///tH[}bg擾
///</summary>
///<returns>tH[}bgԋp</returns>
///----------------------------------------------------------------------
uint32_t Texture::GetFormat()
{
	return mFormat;
}

///----------------------------------------------------------------------
///<summary>
///obt@TCY擾
///</summary>
///<returns>obt@TCYԋp</returns>
///----------------------------------------------------------------------
uint32_t Texture::GetSize()
{
	return mSize;
}

///----------------------------------------------------------------------
///<summary>
///obt@擾
///</summary>
///<returns>obt@|C^ԋp</returns>
///----------------------------------------------------------------------
char* Texture::GetPixels()
{
	return mpBuffers;
}

///----------------------------------------------------------------------
///<summary>
///w肳ꂽCfbNX̃obt@̒l擾
///</summary>
///<param name="index">擾obt@̃CfbNX</param>
///<returns>w肳ꂽCfbNX̃obt@̒lԋp</returns>
///----------------------------------------------------------------------
char Texture::GetPixel( uint32_t index )
{
	assert( index < mSize );
	return mpBuffers[index];
}

///----------------------------------------------------------------------
///<summary>
///t@C擾
///</summary>
///<return>t@Cԋp</returns>
///----------------------------------------------------------------------
std::string Texture::GetName()
{
	return mName;
}

///----------------------------------------------------------------------
///<summary>
///~bv}bv擾
///</summary>
///<reutrns>~bv}bvԋp</returns>
///----------------------------------------------------------------------
uint32_t Texture::GetNumMipmaps()
{
	return mNumMipmaps;
}

Texture& Texture::operator = ( const Texture &texture )
{
	mName = texture.mName;
	mWidth = texture.mWidth;
	mHeight = texture.mHeight;
	mFormat = texture.mFormat;
	mBitsPerPixel = texture.mBitsPerPixel;
	mNumMipmaps = texture.mNumMipmaps;

	Resize( texture.mSize );
	for( uint32_t i=0; i<texture.mSize; i++ )
	{
		mpBuffers[i] = texture.mpBuffers[i];
	}

	return (*this);
}

/////////////////////////////////////////////////////////////////////////
// TextureLoaderBase class
/////////////////////////////////////////////////////////////////////////

///----------------------------------------------------------------------
///<summary>
///RXgN^
///</summary>
///----------------------------------------------------------------------
TextureLoaderBase::TextureLoaderBase()
{
	mpFile = null;
	mBufSize = 0;
	mCurSize = 0;
	mRestSize = 0;
	mpBuf = null;
}

///---------------------------------------------------------------------
///<summary>
///fXgN^
///</summary>
///---------------------------------------------------------------------
TextureLoaderBase::~TextureLoaderBase()
{
	Release();
}

///---------------------------------------------------------------------
///<summary>
///
///</summary>
///---------------------------------------------------------------------
void TextureLoaderBase::Release()
{
	SAFE_DELETE_ARRAY( mpBuf );
	mBufSize = 0;
	mCurSize = 0;
	mRestSize = 0;
	mpFile = null;
}

///---------------------------------------------------------------------
///<summary>
///}WbN`FbN
///</summary>
///<param name="pFilename">}WbN`FbNt@C</param>
///<returns>茋ʂԋp</returns>
///---------------------------------------------------------------------
bool TextureLoaderBase::CheckMagic( const char* pFilename )
{
	/* hNXŏ */
	return true;
}

///---------------------------------------------------------------------
///<summary>
///t@C[h
///</summary>
///<param name="pFilename">[ht@C</param>
///<returns>[hɐꍇtrueԋp</returns>
///---------------------------------------------------------------------
bool TextureLoaderBase::Load( const char *pFilename )
{
	bool result = CheckMagic( pFilename );
	if ( result )
	{
		mpFile = fopen( pFilename, "rb" );
		if ( !mpFile )
		{
			ELOG( "Error : Couldn't open file - %s", pFilename );
			return false;
		}

		std::string name = std::string( pFilename );
		
		fseek( mpFile, 0, SEEK_END );
		mBufSize = static_cast< uint32_t >( ftell( mpFile ) );

		fseek( mpFile, 0, SEEK_SET );

		try
		{
			mpBuf = new char [mBufSize+1];
		}
		catch( std::bad_alloc ba )
		{
			ELOG( "Error : Memory allocate failed. reason = %s", ba.what() );
			return false;
		}

		memset( mpBuf, 0, mBufSize );
		mpBuf[mBufSize] = '\0';

		mCurSize = 0;
		mRestSize = mBufSize;
	}
	return result;
}

///----------------------------------------------------------------------
///<summary>
///t@C[hǂ肷
///</summary>
///<returns>
///[h̏ꍇtrueԋp
///[hI܂͎sꍇfalseԋp.
///</returns>
///----------------------------------------------------------------------
bool TextureLoaderBase::IsLoading()
{
	if ( !mpFile )
	{
		ELOG( "Error : File is not opened." );
		return false;
	}

	uint32_t size = ( mRestSize > 131072 ) ? 131072 : mRestSize; // 128[byte] * 1024 = 131072[byte]
	fread( &mpBuf[ mCurSize ], sizeof(char), size, mpFile );

	mCurSize += size;
	mRestSize -= size;

	if ( mRestSize == 0 )
	{
		DLOG( "Info : Load Finished!" );
		fclose( mpFile );
		return false;
	}

	return true;
}

///----------------------------------------------------------------------
///<summary>
///eNX`擾
///</summary>
///<returns>eNX`ԋp</returns>
///----------------------------------------------------------------------
ITexture* TextureLoaderBase::GetTexture()
{
	return &mTexture;
}

/////////////////////////////////////////////////////////////////////////
// TextureLoaderFactory class
/////////////////////////////////////////////////////////////////////////

ITextureLoader* TextureLoaderFactory::Create( uint32_t type )
{
	ITextureLoader* pLoader = null;
	switch( type )
	{
	case TEXTURE_TYPE_BMP:
		break;

	case TEXTURE_TYPE_TGA:
		break;

	case TEXTURE_TYPE_DDS:
		pLoader = new TextureLoaderDDS();
		break;

	default:
		break;
	}
	return pLoader;
}


} // namespace Schaft
} // namespace Asura
} // namespace Graphics
