//----------------------------------------------------------------------
// File : ASTexture.h
// Desc : Asura Texture
// Copyright(c) 2010 Project Asura All right reserved.
//----------------------------------------------------------------------

//----------------------------------------------------------------------
// History
//----------------------------------------------------------------------
// <---> 2010/11/28 [Pocol] VK쐬.
// <End>

#ifndef __ASURA_TEXTURE_IMPL_H__
#define __ASURA_TEXTURE_IMPL_H__

// Disable Warning C4996
#ifndef _CRT_SECURE_CPP_OVERLOAD_STANDARD_NAMES
#define _CRT_SECURE_CPP_OVERLOAD_STANDARD_NAMES 1
#endif
#ifndef _CRT_SECURE_CPP_OVERLOAD_SECURE_NAMES
#define _CRT_SECURE_CPP_OVERLOAD_SECURE_NAMES 1
#endif
#ifndef _CRT_SECURE_NO_WARNINGS
#define _CRT_SECURE_NO_WARNINGS 1
#endif

//----------------------------------------------------------------------
// Includes
//----------------------------------------------------------------------
#include "AsTexture.h"
#include <cstdio>
#include <string>

namespace Schaft {
namespace Asura {
namespace Graphics {

////////////////////////////////////////////////////////////////////////
// Texture class
////////////////////////////////////////////////////////////////////////
class Texture : public ITexture
{
	//------------------------------------------------------------------
	// list of friend class
	//------------------------------------------------------------------
	/* NOTHING */

public:
	//------------------------------------------------------------------
	// public variables
	//------------------------------------------------------------------
	/* NOTHING */

	//------------------------------------------------------------------
	// public methods
	//------------------------------------------------------------------
	Texture();
	virtual ~Texture();
	void Release();
	void Resize( uint32_t size );
	void SetPixel( uint32_t index, char value );
	void SetPixels( char* pBuffer );
	void SetWidth( uint32_t width );
	void SetHeight( uint32_t height );
	void SetBitsPerPixel( uint16_t bpp );
	void SetFormat( uint32_t format );
	void SetName( const std::string &name );
	void SetNumMipmaps( uint32_t num_mipmaps );

	uint32_t GetWidth();
	uint32_t GetHeight();
	uint16_t GetBitsPerPixel();
	uint32_t GetFormat();
	uint32_t GetSize();
	char* GetPixels();
	char  GetPixel( uint32_t index );
	std::string GetName();
	uint32_t GetNumMipmaps();

	Texture& operator = ( const Texture& texture );

protected:
	//------------------------------------------------------------------
	// protected variables
	//------------------------------------------------------------------
	uint32_t mWidth;
	uint32_t mHeight;
	uint32_t mFormat;
	uint16_t mBitsPerPixel;
	uint32_t mSize;
	char* mpBuffers;
	uint32_t mNumMipmaps;
	std::string mName;

	//-------------------------------------------------------------------
	// protected methods
	//-------------------------------------------------------------------
	/* NOTHING */

private:
	//-------------------------------------------------------------------
	// private variables
	//-------------------------------------------------------------------
	/* NOTHING */

	//-------------------------------------------------------------------
	// private methods
	//-------------------------------------------------------------------
	/* NOTHING */
};

/////////////////////////////////////////////////////////////////////////
// TextureLoaderBase class
/////////////////////////////////////////////////////////////////////////
class TextureLoaderBase : public ITextureLoader
{
	//-------------------------------------------------------------------
	// list of friend classes
	//-------------------------------------------------------------------
	/* NOTHING */

public:
	//-------------------------------------------------------------------
	// public variables
	//-------------------------------------------------------------------
	/* NOTHING */

	//-------------------------------------------------------------------
	// public methods
	//-------------------------------------------------------------------
	TextureLoaderBase();
	virtual ~TextureLoaderBase();
	virtual bool Load( const char* pFilename );
	virtual bool IsLoading();
	virtual bool CheckMagic( const char* pFilename );
	virtual bool Parse() = 0;
	virtual void Release();
	ITexture* GetTexture();

protected:
	//-------------------------------------------------------------------
	// protected variables
	//-------------------------------------------------------------------
	Texture  mTexture;
	FILE*    mpFile;
	uint32_t mBufSize;
	uint32_t mCurSize;
	uint32_t mRestSize;
	char*    mpBuf;

	//-------------------------------------------------------------------
	// protected methods
	//-------------------------------------------------------------------
	/* NOTHING */

private:
	//-------------------------------------------------------------------
	// private variables
	//-------------------------------------------------------------------
	/* NOTHING */

	//-------------------------------------------------------------------
	// private methods
	//-------------------------------------------------------------------
	/* NOTHING */

};

} // namespace Graphics
} // namespace Asura
} // namespace Schaft

#endif // __ASURA_TEXTURE_IMPL_H__