//----------------------------------------------------------------------
// File : AsuraModelUtil.cpp
// Desc : Asura Model Utility
// Copyright(c) Project Asura All right reserved.
//----------------------------------------------------------------------

//----------------------------------------------------------------------
// History
//----------------------------------------------------------------------
// <---> 2010/03/26 [Pocol] VK쐬
// <End>


//----------------------------------------------------------------------
// Includes
//----------------------------------------------------------------------
#include "AsModelUtil.h"


//----------------------------------------------------------------------
// Using Statements
//----------------------------------------------------------------------
using namespace Schaft::Asura;
using namespace Schaft::Asura::Math;
using namespace Schaft::Asura::Graphics;
using namespace Schaft::Asura::Util;


namespace Schaft {
namespace Asura {
namespace Util {

///----------------------------------------------------------------------
///<summary>
///GLG[邩ǂ肷
///<summary>
///<returns>GLG[ꍇtrueԋp</returns>
///----------------------------------------------------------------------
bool IsGLError()
{
	bool result = false;

	GLenum error = glGetError();
	if ( error != GL_NO_ERROR )
	{
		result = true;
		switch( error )
		{
			case GL_INVALID_ENUM:
				ELOG( "GLError : Invalid Enum." );
				break;

			case GL_INVALID_VALUE:
				ELOG( "GLError : Invalid Value." );
				break;

			case GL_INVALID_OPERATION:
				ELOG( "GLError : Invalid Operation." );
				break;

			case GL_STACK_OVERFLOW:
				ELOG( "GLError : Stack Overflow." );
				break;

			case GL_STACK_UNDERFLOW:
				ELOG( "GLError : Stack Underflow." );
				break;

			case GL_OUT_OF_MEMORY:
				ELOG( "GLError : Out Of Memory." );
				break;

			default:
				ELOG( "GLError : error code=%d", error );
				break;
		}
	}
	return result;	
}

////////////////////////////////////////////////////////////////////////
// ModelData class
////////////////////////////////////////////////////////////////////////

///---------------------------------------------------------------------
///<summary>
///RXgN^
///</summary>
///---------------------------------------------------------------------
ModelData::ModelData()
{
    mMeshes.clear();
    mpInterface = null;
    mpLoader = null;
}

///---------------------------------------------------------------------
///<summary>
///fXgN^
///</summary>
///---------------------------------------------------------------------
ModelData::~ModelData()
{
    Release();
}

///---------------------------------------------------------------------
///<summary>
///Xt@C烂f[h
///</summary>
///<param name="filename">[ht@C</param>
///<returns>[hɐtrueԋp</returns>
///---------------------------------------------------------------------
bool ModelData::LoadFromX( const char *filename )
{
    bool result = false;

    // f[_[𐶐
    if ( !CreateLoader() )
    {
        ELOG( "Error : LoadFromX() Failed. reason = Model Loader is not exist." );
        return false;
    }

    // ǂݍݍς݃fj
    ReleaseInterface();

    // fǂݍ
    bool isLoaded = false;
    result = mpLoader->Load( filename );
    if ( result )
    {
        for( ;; )
        {
            // [hI܂Ń[v
            if( !mpLoader->IsLoading() )
            {
                //@
                mpLoader->Parse();

                //@C^tF[X擾
                mpInterface = mpLoader->GetModel();

                //@擾łĂ΃tO𗧂Ă
                if ( mpInterface != null )
                {
                    isLoaded = true;
                }
                break;
            }
        }
    }

    // f̒_obt@̐s
    if ( isLoaded )
    {
        // _obt@ƃCfbNXobt@i[p̃m
        mMeshes.resize( mpInterface->GetNumMeshes() );

        // bV̐s
        for( uint32_t i=0; i<mpInterface->GetNumMeshes(); ++i )
        {
            // obt@̐
            if ( !CreateBuffer( mMeshes[ i ], mpInterface->GetMesh( i ) ) )
            {
                ELOG( "Error : Vertex Buffer Create Failed. index = %d", i );
                
                for( uint32_t j=0; j<i; ++j )
                {
                    // s쐬̃obt@
                    ReleaseBuffer( mMeshes[j] );
                }

                result = false;
                break;
            }
        }
    }   

    return result;
}

///---------------------------------------------------------------------
///<summary>
///f[_𐶐
///</summary>
///<returns>
///ς݂܂͐ꂽꍇtrueԋpC
///ɎsƂ̂falseԋp
///</returns>
///---------------------------------------------------------------------
bool ModelData::CreateLoader()
{
    bool result = true;

    // f[_[̐
    if ( mpLoader == null )
    {
        mpLoader = ModelLoaderFactory::Create( ModelLoaderFactory::MODEL_TYPE_X );
        if ( mpLoader == null )
        {
            ELOG( "Error : ModelLoaderFactoryCreate() Failed." );
            result = false;
        }
        else
        {
            result = true;
        }
    }

    return result;
}

///---------------------------------------------------------------------
///<summary>
///f[_[
///</summary>
///---------------------------------------------------------------------
void ModelData::ReleaseLoader()
{
    if( mpLoader )
    {
        mpLoader->Release();
        delete mpLoader;
        mpLoader = null;
    }
}

///---------------------------------------------------------------------
///<summary>
///fC^tF[X
///</summary>
///---------------------------------------------------------------------
void ModelData::ReleaseInterface()
{
    if ( mpInterface )
    {
        mpInterface->Release();
        delete mpInterface;
        mpInterface = null;
    }
}

///---------------------------------------------------------------------
///<summary>
///obt@𐶐
///</summary>
///---------------------------------------------------------------------
bool ModelData::CreateBuffer( MeshData &data, IMesh *pMesh )
{
    // C^tF[XƂ̂ݏ
    if ( pMesh == null )
    {
        ELOG( "Error : Model Interface is not available." );
        assert( pMesh != null );
        return false;
    }

    bool result = true;

    // [NA
    std::memset( data.mVB, 0, sizeof(uint32_t) * NUM_VERTEX_DATA_TYPE );

    // ʒuW
    if ( pMesh->GetNumPositions() > 0 )
    {
        glGenBuffers( 1, &data.mVB[ VERTEX_DATA_POSITION ] );
        glBindBuffer( GL_ARRAY_BUFFER, data.mVB[ VERTEX_DATA_POSITION ] );
        glBufferData( GL_ARRAY_BUFFER, sizeof( Vector3 ) * pMesh->GetNumPositions(), pMesh->GetPositions()[0], GL_STATIC_DRAW );
    }

    // @xNg
    if ( pMesh->GetNumNormals() > 0 )
    {
        glGenBuffers( 1, &data.mVB[ VERTEX_DATA_NORMAL ] );
        glBindBuffer( GL_ARRAY_BUFFER, data.mVB[ VERTEX_DATA_NORMAL ] );
        glBufferData( GL_ARRAY_BUFFER, sizeof( Vector3 ) * pMesh->GetNumNormals(), pMesh->GetNormals()[0], GL_STATIC_DRAW );
    }

    // _J[
    if ( pMesh->GetNumColors() > 0 )
    {
        glGenBuffers( 1, &data.mVB[ VERTEX_DATA_COLOR ] );
        glBindBuffer( GL_ARRAY_BUFFER, data.mVB[ VERTEX_DATA_COLOR ] );
        glBufferData( GL_ARRAY_BUFFER, sizeof( UByte4 ) * pMesh->GetNumColors(), pMesh->GetColors()[0], GL_STATIC_DRAW );
    }

    // eNX`W
    if ( pMesh->GetNumTexCoords() > 0 )
    {
        glGenBuffers( 1, &data.mVB[ VERTEX_DATA_TEXCOORD ] );
        glBindBuffer( GL_ARRAY_BUFFER, data.mVB[ VERTEX_DATA_TEXCOORD ] );
        glBufferData( GL_ARRAY_BUFFER, sizeof( Vector2 ) * pMesh->GetNumTexCoords(), pMesh->GetTexCoords()[0], GL_STATIC_DRAW );
    }

    // ڃxNg
    if ( pMesh->GetNumTangents() > 0 )
    {
        glGenBuffers( 1, &data.mVB[ VERTEX_DATA_TANGENT ] );
        glBindBuffer( GL_ARRAY_BUFFER, data.mVB[ VERTEX_DATA_TANGENT ] );
        glBufferData( GL_ARRAY_BUFFER, sizeof( Vector3 ) * pMesh->GetNumTangents(), pMesh->GetTangents()[0], GL_STATIC_DRAW );
    }

    // ]@xNg
    if ( pMesh->GetNumBinormals() > 0 )
    {
        glGenBuffers( 1, &data.mVB[ VERTEX_DATA_BINORMAL ] );
        glBindBuffer( GL_ARRAY_BUFFER, data.mVB[ VERTEX_DATA_BINORMAL ] );
        glBufferData( GL_ARRAY_BUFFER, sizeof( Vector3 ) * pMesh->GetNumBinormals(), pMesh->GetBinormals()[0], GL_STATIC_DRAW );
    }

    // XLCfbNX
    if ( pMesh->GetNumSkinIndices() > 0 )
    {
        glGenBuffers( 1, &data.mVB[ VERTEX_DATA_SKIN_INDEX ] );
        glBindBuffer( GL_ARRAY_BUFFER, data.mVB[ VERTEX_DATA_SKIN_INDEX ] );
        glBufferData( GL_ARRAY_BUFFER, sizeof( Short4 ) * pMesh->GetNumSkinIndices(), pMesh->GetSkinIndices()[0], GL_STATIC_DRAW );
    }

    // XLEFCg
    if ( pMesh->GetNumSkinWeights() > 0 )
    {
        glGenBuffers( 1, &data.mVB[ VERTEX_DATA_SKIN_WEIGHT ] );
        glBindBuffer( GL_ARRAY_BUFFER, data.mVB[ VERTEX_DATA_SKIN_WEIGHT ] );
        glBufferData( GL_ARRAY_BUFFER, sizeof( Vector4 ) * pMesh->GetNumSkinWeights(), pMesh->GetSkinWeights()[0], GL_STATIC_DRAW );
    }
    
    // m
    data.mSubsets.resize( pMesh->GetNumSubsets() );
    
    for( uint32_t i=0; i<pMesh->GetNumSubsets(); ++i )
    {
        IMeshSubset* pSubset = pMesh->GetSubset( i );
        if ( pSubset == null )
        {
            continue;
        }

        if ( pMesh->GetNumIndices() > 0 )
        {
            uint32_t numIndices = pSubset->GetNumFace() * 3;
            uint32_t startIndex = pSubset->GetFaceIndex() * 3;

            // TuZbg
            data.mSubsets[i].mCount = pSubset->GetNumFace() * 3;
            data.mSubsets[i].mMatIdx = pSubset->GetMaterialIndex();
            glGenBuffers( 1, &data.mSubsets[i].mIB );
            glBindBuffer( GL_ELEMENT_ARRAY_BUFFER, data.mSubsets[i].mIB );
            glBufferData( GL_ELEMENT_ARRAY_BUFFER, sizeof( uint32_t ) * numIndices, &pMesh->GetIndices()[ startIndex ], GL_STATIC_DRAW );
        }
    }

    // GLG[`FbN
    if ( IsGLError() )
    {
        ELOG( "Error : GL Error Occured." );
        result = false;
    }

    return result;
}

///---------------------------------------------------------------------
///<summary>
///obt@
///</summary>
///---------------------------------------------------------------------
void ModelData::ReleaseBuffer( MeshData &data )
{
    // ʒuW
    if ( data.mVB[ VERTEX_DATA_POSITION ] > 0 )
    {
        glDeleteBuffers( 1, &data.mVB[ VERTEX_DATA_POSITION ] );
        data.mVB[ VERTEX_DATA_POSITION ] = 0;
    }

    // @xNg
    if ( data.mVB[ VERTEX_DATA_NORMAL ] > 0 )
    {
        glDeleteBuffers( 1, &data.mVB[ VERTEX_DATA_NORMAL ] );
        data.mVB[ VERTEX_DATA_NORMAL ] = 0;
    }

    // _J[
    if ( data.mVB[ VERTEX_DATA_COLOR ] > 0 )
    {
        glDeleteBuffers( 1, &data.mVB[ VERTEX_DATA_COLOR ] );
        data.mVB[ VERTEX_DATA_COLOR ] = 0;
    }

    // eNX`W
    if ( data.mVB[ VERTEX_DATA_TEXCOORD ] > 0 )
    {
        glDeleteBuffers( 1, &data.mVB[ VERTEX_DATA_TEXCOORD ] );
        data.mVB[ VERTEX_DATA_TEXCOORD ] = 0;
    }

    // ڃxNg
    if ( data.mVB[ VERTEX_DATA_TANGENT ] > 0 )
    {
        glDeleteBuffers( 1, &data.mVB[ VERTEX_DATA_TANGENT ] );
        data.mVB[ VERTEX_DATA_TANGENT ] = 0;
    }

    // ]@xNg
    if ( data.mVB[ VERTEX_DATA_BINORMAL ] > 0 )
    {
        glDeleteBuffers( 1, &data.mVB[ VERTEX_DATA_BINORMAL ] );
        data.mVB[ VERTEX_DATA_BINORMAL ] = 0;
    }

    // XLCfbNX
    if ( data.mVB[ VERTEX_DATA_SKIN_INDEX ] > 0 )
    {
        glDeleteBuffers( 1, &data.mVB[ VERTEX_DATA_SKIN_INDEX ] );
        data.mVB[ VERTEX_DATA_SKIN_INDEX ] = 0;
    }

    // XLEFCg
    if ( data.mVB[ VERTEX_DATA_SKIN_WEIGHT ] > 0 )
    {
        glDeleteBuffers( 1, &data.mVB[ VERTEX_DATA_SKIN_WEIGHT ] );
        data.mVB[ VERTEX_DATA_SKIN_WEIGHT ] = 0;
    }

    // TuZbg
    for( uint32_t i=0; i<data.mSubsets.size(); ++i )
    {
        if ( data.mSubsets[i].mIB > 0 )
        {
            glDeleteBuffers( 1, &data.mSubsets[i].mIB );
            data.mSubsets[i].mIB = 0;
            data.mSubsets[i].mCount = 0;
            data.mSubsets[i].mMatIdx = 0;
        }
    }
    data.mSubsets.clear();

    // GLG[`FbN
    if ( IsGLError() )
    {
       ELOG( "Error : GL Error Occured." );
    }
}

///---------------------------------------------------------------------
///<summary>
///
///</summary>
///---------------------------------------------------------------------
void ModelData::Release()
{
    // f[_[
    ReleaseLoader();

    // fC^[tFCX
    ReleaseInterface();  

    // obt@
    for( uint32_t i=0; i<mMeshes.size(); ++i )
    {
        ReleaseBuffer( mMeshes[i] );
    }
    mMeshes.clear();
}

///---------------------------------------------------------------------
///<summary>
///f`悷
///</summary>
///---------------------------------------------------------------------
void ModelData::Draw()
{
    // ebV`悷
    for( uint32_t i=0; i<mpInterface->GetNumMeshes(); ++i )
    {
        DrawMesh( mMeshes[i], mpInterface->GetMesh(i)->GetMaterials() );
    }
}

///---------------------------------------------------------------------
///<summary>
///bV`悷
///</summary>
///---------------------------------------------------------------------
void ModelData::DrawMesh( const MeshData &mesh, const std::vector< IMaterial* > &materials )
{
    // ʒuW
    if ( mesh.mVB[ VERTEX_DATA_POSITION ] > 0 )
    {
        glEnableClientState( GL_VERTEX_ARRAY );
        glBindBuffer( GL_ARRAY_BUFFER, mesh.mVB[ VERTEX_DATA_POSITION ] );
        glVertexPointer( 3, GL_FLOAT, 0, null );
    }

    // @xNg
    if ( mesh.mVB[ VERTEX_DATA_NORMAL ] > 0 )
    {
        glEnableClientState( GL_NORMAL_ARRAY );
        glBindBuffer( GL_ARRAY_BUFFER, mesh.mVB[ VERTEX_DATA_NORMAL ] );
        glNormalPointer( GL_FLOAT, 0, null );
    }

    // _J[
    if ( mesh.mVB[ VERTEX_DATA_COLOR ] > 0 )
    {
        glEnableClientState( GL_COLOR_ARRAY );
        glBindBuffer( GL_ARRAY_BUFFER, mesh.mVB[ VERTEX_DATA_COLOR ] );
        glColorPointer( 4, GL_UNSIGNED_BYTE, 0, null );
    }

    // eNX`W
    if ( mesh.mVB[ VERTEX_DATA_TEXCOORD ] > 0 )
    {
        glClientActiveTexture( GL_TEXTURE0 );
        glEnableClientState( GL_TEXTURE_COORD_ARRAY );
        glBindBuffer( GL_ARRAY_BUFFER, mesh.mVB[ VERTEX_DATA_TEXCOORD ] );
        glTexCoordPointer( 2, GL_FLOAT, 0, null );
    }
 
    // ڃxNg
    if ( mesh.mVB[ VERTEX_DATA_TANGENT ] > 0 )
    {
        glClientActiveTexture( GL_TEXTURE1 );
        glEnableClientState( GL_TEXTURE_COORD_ARRAY );
        glBindBuffer( GL_ARRAY_BUFFER, mesh.mVB[ VERTEX_DATA_TANGENT ] );
        glTexCoordPointer( 3, GL_FLOAT, 0, null );
    }

    // ]@xNg
    if ( mesh.mVB[ VERTEX_DATA_BINORMAL ] > 0 )
    {
        glClientActiveTexture( GL_TEXTURE2 );
        glEnableClientState( GL_TEXTURE_COORD_ARRAY );
        glBindBuffer( GL_ARRAY_BUFFER, mesh.mVB[ VERTEX_DATA_BINORMAL ] );
        glTexCoordPointer( 3, GL_FLOAT, 0, null );
    }

    // XLCfbNX
    if ( mesh.mVB[ VERTEX_DATA_SKIN_INDEX ] > 0 )
    {
        glClientActiveTexture( GL_TEXTURE3 );
        glEnableClientState( GL_TEXTURE_COORD_ARRAY );
        glBindBuffer( GL_ARRAY_BUFFER, mesh.mVB[ VERTEX_DATA_SKIN_INDEX ] );
        glTexCoordPointer( 4, GL_SHORT, 0, null );
    }

    // XLEFCg
    if ( mesh.mVB[ VERTEX_DATA_SKIN_WEIGHT ] > 0 )
    {
        glClientActiveTexture( GL_TEXTURE4 );
        glEnableClientState( GL_TEXTURE_COORD_ARRAY );
        glBindBuffer( GL_ARRAY_BUFFER, mesh.mVB[ VERTEX_DATA_SKIN_WEIGHT ] );
        glTexCoordPointer( 4, GL_FLOAT, 0, null );
    }

    // }eATCY`FbN
    bool isUseMaterial = false;
    if ( materials.size() > 0 )
    {
        isUseMaterial = true;
    };

    for( size_t i=0; i<mesh.mSubsets.size(); ++i )
    {
        // }eA̐ݒ
        if ( isUseMaterial )
        {
            // dȂ̂GPUłglstate.diffusep邱
            glMaterialfv( GL_FRONT_AND_BACK, GL_DIFFUSE,   materials[ mesh.mSubsets[i].mMatIdx ]->GetDiffuse() );
            glMaterialfv( GL_FRONT_AND_BACK, GL_SPECULAR,  materials[ mesh.mSubsets[i].mMatIdx ]->GetSpecular() );
            glMaterialfv( GL_FRONT_AND_BACK, GL_EMISSION,  materials[ mesh.mSubsets[i].mMatIdx ]->GetEmissive() );
            glMaterialf(  GL_FRONT_AND_BACK, GL_SHININESS, materials[ mesh.mSubsets[i].mMatIdx ]->GetPower() );
        }
       
        // TuZbg`
        if ( mesh.mSubsets[i].mIB > 0 )
        {
            glBindBuffer( GL_ELEMENT_ARRAY_BUFFER, mesh.mSubsets[i].mIB );
            glDrawElements( GL_TRIANGLES, mesh.mSubsets[i].mCount, GL_UNSIGNED_INT, null );
        }
    }

    // obt@֘AtO
    glBindBuffer( GL_ELEMENT_ARRAY_BUFFER, 0 );
    glBindBuffer( GL_ARRAY_BUFFER, 0 );

    // XLEFCg
    if ( mesh.mVB[ VERTEX_DATA_SKIN_WEIGHT ] > 0 )
    {
        glClientActiveTexture( GL_TEXTURE4 );
        glDisableClientState( GL_TEXTURE_COORD_ARRAY );
    }

    // XLCfbNX
    if ( mesh.mVB[ VERTEX_DATA_SKIN_INDEX ] > 0 )
    {
        glClientActiveTexture( GL_TEXTURE3 );
        glDisableClientState( GL_TEXTURE_COORD_ARRAY );
    }

    // ]@xNg
    if ( mesh.mVB[ VERTEX_DATA_BINORMAL ] > 0 )
    {
        glClientActiveTexture( GL_TEXTURE2 );
        glDisableClientState( GL_TEXTURE_COORD_ARRAY );
    }

    // ڃxNg
    if ( mesh.mVB[ VERTEX_DATA_TANGENT ] > 0 )
    {
        glClientActiveTexture( GL_TEXTURE1 );
        glDisableClientState( GL_TEXTURE_COORD_ARRAY );
    }

    // eNX`W
    if ( mesh.mVB[ VERTEX_DATA_TEXCOORD ] > 0 )
    {
        glClientActiveTexture( GL_TEXTURE0 );
        glDisableClientState( GL_TEXTURE_COORD_ARRAY );
    }

    // _J[
    if ( mesh.mVB[ VERTEX_DATA_COLOR ] > 0 )
    {
        glDisableClientState( GL_COLOR_ARRAY );
    }

    // @xNg
    if ( mesh.mVB[ VERTEX_DATA_NORMAL ] > 0 )
    {
        glDisableClientState( GL_NORMAL_ARRAY );
    }

    // ʒuW
    if ( mesh.mVB[ VERTEX_DATA_POSITION ] > 0 )
    {
        glDisableClientState( GL_VERTEX_ARRAY );
    }

#if defined(DEBUG) || defined(_DEBUG)
    // GLG[`FbN
    if ( IsGLError() )
    {
        ELOG( "Error : GL Error Occured." );
    }
#endif
}

////////////////////////////////////////////////////////////////////////
// SkinningModelData class
////////////////////////////////////////////////////////////////////////

///---------------------------------------------------------------------
///<summary>
///RXgN^
///</summary>
///---------------------------------------------------------------------
SkinningModelData::SkinningModelData()
: ModelData()
{
    mpPlayer = null;
    mRoot.Identity();
}

///---------------------------------------------------------------------
///<summary>
///fXgN^
///</summary>
///---------------------------------------------------------------------
SkinningModelData::~SkinningModelData()
{
    Release();
}

///---------------------------------------------------------------------
///<summary>
///Xt@C烂f[h
///</summary>
///<param name="filename">[h郂ft@C</param>
///<return>[hɐtrueԋp</return>
///---------------------------------------------------------------------
bool SkinningModelData::LoadFromX(const char *filename)
{
    bool result = false;

    // f[_[𐶐
    if ( !CreateLoader() )
    {
        ELOG( "Error : LoadFromX() Failed. reason = Model Loader is not exist." );
        return false;
    }

    // ǂݍݍς݃fj
    ReleaseInterface();

    // fǂݍ
    bool isLoaded = false;
    result = mpLoader->Load( filename );
    if ( result )
    {
        for( ;; )
        {
            // [hI܂Ń[v
            if( !mpLoader->IsLoading() )
            {
                //@
                mpLoader->Parse();

                //@C^tF[X擾
                mpInterface = mpLoader->GetModel();

                //@擾łĂ΃tO𗧂Ă
                if ( mpInterface != null )
                {
                    isLoaded = true;
                }
                break;
            }
        }
    }

    // f̒_obt@̐s
    if ( isLoaded )
    {
        // _obt@ƃCfbNXobt@i[p̃m
        mMeshes.resize( mpInterface->GetNumMeshes() );

        // bV̐s
        for( uint32_t i=0; i<mpInterface->GetNumMeshes(); ++i )
        {
            // obt@̐
            if ( !CreateBuffer( mMeshes[ i ], mpInterface->GetMesh( i ) ) )
            {
                ELOG( "Error : Vertex Buffer Create Failed. index = %d", i );
                
                for( uint32_t j=0; j<i; ++j )
                {
                    // s쐬̃obt@
                    ReleaseBuffer( mMeshes[j] );
                }
                ReleaseInterface();
                ReleaseLoader();

                result = false;
                break;
            }
        }

        // Aj[VvC[𐶐
        if ( mpInterface->GetMesh(0)->GetNumBones() > 0 )
        {
            mpPlayer = AnimationPlayerFactory::Create( mpInterface->GetMesh(0)->GetBones() );

            // ꂽAj[VNbvݒ
            if ( mpPlayer != null )
            {
                if ( mpInterface->GetNumClips() > 0 )
                {
                    mpPlayer->SetClip( mpInterface->GetClip(0) );
                    mTimer.Start();
                }
                else
                {
                    ELOG( "Error : AnimationClip is not exist." );

                    Release();
                    result = false;
                }
            }
            else
            {
                ELOG( "Error : AnimationPlayerFactory::Create() Failed." );

                Release();
                result = false;
            }
        }
    }   

    return result;
}

///---------------------------------------------------------------------
///<summary>
///
///</summary>
///---------------------------------------------------------------------
void SkinningModelData::Release()
{
    // f[_[
    ReleaseLoader();

    // fC^[tFCX
    ReleaseInterface();  

    // obt@
    for( uint32_t i=0; i<mMeshes.size(); ++i )
    {
        ReleaseBuffer( mMeshes[i] );
    }
    mMeshes.clear();

    ReleasePlayer();
    mTimer.Reset();
}

///---------------------------------------------------------------------
///<summary>
///Aj[VvC[
///</summary>
///---------------------------------------------------------------------
void SkinningModelData::ReleasePlayer()
{
    if ( mpPlayer )
    {
        mpPlayer->Release();
        delete mpPlayer;
        mpPlayer = null;
    }
}

///---------------------------------------------------------------------
///<summary>
///Aj[VXV
///</summary>
///<param name="isStop">Aj[V~tO</param>
///<param name="speed">ĐXs[hW</param>
///---------------------------------------------------------------------
void SkinningModelData::Update( const bool isStop, const double speed )
{
    if ( mpPlayer != null )
    {
        mpPlayer->Update( 
            static_cast< float >( mTimer.GetElapsedTime() * speed ),
            !isStop, mRoot );
    }
}

///---------------------------------------------------------------------
///<summary>
///Aj[VNbvݒ肷
///</summary>
///<param name="index">fɕtAj[Vf[^̃CfbNX</param>
///---------------------------------------------------------------------
void SkinningModelData::SetClip(const unsigned int index)
{
    if ( 0 <= index && index < mpInterface->GetNumClips() )
    {
        if ( mpPlayer != null )
        {
            mpPlayer->SetClip( mpInterface->GetClip( index ) );
        }
    }
}

///---------------------------------------------------------------------
///<summary>
///f̃[gsݒ肷
///</summary>
///<param name="matrix">ݒ肷郂f̃[gs</param>
///---------------------------------------------------------------------
void SkinningModelData::SetRootMatrix( const Matrix &matrix )
{
    mRoot = matrix;
}

///---------------------------------------------------------------------
///<summary>
///f̃[gs擾
///</summary>
///<return>[gsԋp</returns>
///---------------------------------------------------------------------
Matrix SkinningModelData::GetRootMatrix()
{
    return mRoot;
}

///---------------------------------------------------------------------
///<summary>
///XLjOs擾
///</summary>
///<return>
///XLjOsԋp.
///vC[ĂȂꍇ́CTCY0̃Reiԋp
///</return>
///---------------------------------------------------------------------
std::vector< Matrix > SkinningModelData::GetSkinTransformMatrix()
{
    if ( mpPlayer )
    {
        return mpPlayer->GetSkinTransform();
    }
    std::vector< Matrix > result;
    result.clear();
    return result;
}

} // namespace Util
} // namespace Asura
} // namespace Schaft