//-----------------------------------------------------------------------
// File : DeferredRenderer.h
// Desc : Deferred Renderer
// Copyright(c) Project Asura All right reserved.
//-----------------------------------------------------------------------

//-----------------------------------------------------------------------
// History
//-----------------------------------------------------------------------
// <---> 2011/05/14 [Pocol] VK쐬.
// <End>


//-----------------------------------------------------------------------
// Includes
//-----------------------------------------------------------------------
#include "DeferredRenderer.h"
#include <Cg/cgGL.h>


//----------------------------------------------------------------------
// Using Statements
//----------------------------------------------------------------------
using namespace Schaft::Asura;
using namespace Schaft::Asura::Graphics;
using namespace Schaft::Asura::Math;
using namespace Schaft::Asura::SampleProgram;
using namespace Schaft::Asura::Util;


namespace Schaft {
namespace Asura {
namespace SampleProgram {

//---------------------------------------------------------------------
// Constant Values
//---------------------------------------------------------------------
const int FRAME_BUFFER_WIDTH  = 2048;   // 4096ƃG[ɂȂ̂2048Ƀ_E
const int FRAME_BUFFER_HEIGHT = 2048;

    
///----------------------------------------------------------------------
///<summary>
///ftHgl̐ݒ
///</summary>
///----------------------------------------------------------------------
DeferredRenderer::DeferredRenderer()
{
}

///----------------------------------------------------------------------
///<summary>
///fXgN^
///</summary>
///----------------------------------------------------------------------
DeferredRenderer::~DeferredRenderer()
{
    // Yh~
    Release();
}

///----------------------------------------------------------------------
///<summary>
///
///</summary>
///<returns>ɐtrueԋp</returns>
///----------------------------------------------------------------------
bool DeferredRenderer::Initialize()
{
    bool result = true;

    // f[h
    result = mSaturn.LoadFromX( "Contents/Models/dosei.x" );
    if ( !result )
    {
        ELOG( "Error : Model File Load Failed." );
        return false;
    }
    result = mGround.LoadFromX( "Contents/Models/ground.x" );
    if ( !result )
    {
        ELOG( "Error : Model File Load Failed." );
        return false;
    }

    // Jݒ
    mCamera.SetType( DemoCamera::CAMERA_TYPE_THIRD_PERSON );
    Vector3 pos = Vector3( 0.0f, 0.0f, 5.0f );
    mCamera.SetPosition( pos );

    // GtFNgC^tF[X쐬
    mpEffect = CgEffectFactory::Create();
    if ( mpEffect == null )
    {
        ELOG( "Error : Effect Interface Cerate Failed." );
        return false;
    }

    // GtFNgt@C[h
    if ( !mpEffect->CreateFromFile( "Contents/Effects/DeferredRendering.cgfx", null ) )
    {
        ELOG( "Error : Cg Effect File Load Failed." );
        return false;
    }

    // GtFNgp[^擾
    std::memset( &mParam, 0, sizeof( EffectParam ) );
    mParam.world            = mpEffect->GetParameter( "World" );
    mParam.view             = mpEffect->GetParameter( "View" );
    mParam.projection       = mpEffect->GetParameter( "Projection" );
    mParam.lightPostion     = mpEffect->GetParameter( "LightPosition" );
    mParam.lightColor       = mpEffect->GetParameter( "LightColor" );
    mParam.cameraPosition   = mpEffect->GetParameter( "CameraPosition" );
    mParam.positionSmp      = mpEffect->GetParameter( "PositionSmp" );
    mParam.normalDepthSmp   = mpEffect->GetParameter( "NormalDepthSmp" );
    mParam.diffuseAlbedoSmp = mpEffect->GetParameter( "DiffuseAlbedoSmp" );

    // MRT
    mPrevWidth  = DemoApp::GetInstance()->GetWidth();
    mPrevHeight = DemoApp::GetInstance()->GetHeight();
    result = InitializeMRT( FRAME_BUFFER_WIDTH, FRAME_BUFFER_HEIGHT );
    if ( !result )
    {
        ELOG( "Error : IntializeMRT() Failed." );
    }

    // r[|[gݒ
    glViewport( 0, 0, mPrevWidth, mPrevHeight );

    // wʃJOL
    glEnable( GL_CULL_FACE );
    glCullFace( GL_BACK );

    assert( !GL_ERROR_CHECK );

    // Cg̈ʒuƐF͂ĂƂ`ɐݒ
    mLightPosition[ 0 ] = Vector4( 0.0f, 2.0f, -2.0f, 1.0f );
    mLightColor[ 0 ]    = Vector3( 1.0f, 1.0f, 1.0f );

    mLightPosition[ 1 ] = Vector4( 0.0f, 2.0f, 2.0f, 1.0f );
    mLightColor[ 1 ]    = Vector3( 1.0f, 0.0f, 0.0f );

    mLightPosition[ 2 ] = Vector4( 2.0f, 2.0f, 0.0f, 1.0f );
    mLightColor[ 2 ]    = Vector3( 0.0f, 1.0f, 0.0f );

    mLightPosition[ 3 ] = Vector4( -2.0f, 2.0f, 0.0f, 1.0f );
    mLightColor[ 3 ]    = Vector3( 0.0f, 0.0f, 1.0f );

    mLightPosition[ 4 ] = Vector4( 0.0f, 2.0f, 0.0f, 1.0f );
    mLightColor[ 4 ]    = Vector3( 1.0f, 1.0f, 0.0f );

    mLightPosition[ 5 ] = Vector4( -2.0f, 0.0f, 0.0f, 1.0f );
    mLightColor[ 5 ]    = Vector3( 1.0f, 0.0f, 1.0f );

    mLightPosition[ 6 ] = Vector4( 2.0f, 0.0f, 0.0f, 1.0f );
    mLightColor[ 6 ]    = Vector3( 0.0f, 1.0f, 1.0f );

    mLightPosition[ 7 ] = Vector4( 0.0f, 0.0f, 2.0f, 1.0f );
    mLightColor[ 7 ]    = Vector3( 1.0f, 0.5f, 0.0f );

    mLightPosition[ 8 ] = Vector4( 0.0f, 0.0f, 2.0f, 1.0f );
    mLightColor[ 8 ]    = Vector3( 0.5f, 1.0f, 0.0f );

    return result;
}

///----------------------------------------------------------------------
///<summary>
///MRT̏
///</summary>
///<param name="width">t[obt@̕</param>
///<param name="height">t[obt@̍</param>
///<returns>ɐtrueԋp</returns>
///----------------------------------------------------------------------
bool DeferredRenderer::InitializeMRT( const int width, const int height )
{
    //@GLEW̏
    if ( glewInit() != GLEW_OK )
    {
        ELOG( "Error : glewInit() Failed." );
        return false;
    }

    // _eNX`g邩`FbN
    if ( !GLEW_ARB_texture_float )
    {
        ELOG( "Error : ARB_texture_float is not supported." );
        return false;
    }

    //@eNX`3
	glGenTextures( NUM_GBUFFER, mTexture );

    //@3̐ݒĂ
    glEnable( GL_TEXTURE_2D );
    for( int i=0; i<NUM_GBUFFER; i++ )
    {
        glBindTexture( GL_TEXTURE_2D, mTexture[i] );
        glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
    	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
    	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE );
    	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE );
        glTexImage2D( GL_TEXTURE_2D, 0, GL_RGBA32F_ARB, width, height, 0, GL_RGBA, GL_UNSIGNED_BYTE, 0 );
        glBindTexture( GL_TEXTURE_2D, 0 );
    }
    glDisable( GL_TEXTURE_2D );
    assert( !GL_ERROR_CHECK );

    //@_[obt@̐ݒ
    glGenRenderbuffersEXT( 1, &mRenderBuffer );
    glBindRenderbufferEXT( GL_RENDERBUFFER_EXT, mRenderBuffer );
    glRenderbufferStorageEXT( GL_RENDERBUFFER_EXT, GL_DEPTH_COMPONENT24, width, height );

    //@t[obt@̐ݒ
    glGenFramebuffersEXT( 1, &mFrameBuffer );
    glBindFramebufferEXT( GL_FRAMEBUFFER_EXT, mFrameBuffer );
    glFramebufferTexture2DEXT( GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT, GL_TEXTURE_2D, mTexture[0], 0 );
    glFramebufferTexture2DEXT( GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT1_EXT, GL_TEXTURE_2D, mTexture[1], 0 );
    glFramebufferTexture2DEXT( GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT2_EXT, GL_TEXTURE_2D, mTexture[2], 0 );
    glFramebufferRenderbufferEXT( GL_FRAMEBUFFER_EXT, GL_DEPTH_ATTACHMENT_EXT, GL_RENDERBUFFER_EXT, mRenderBuffer );
    glBindFramebufferEXT( GL_FRAMEBUFFER_EXT, 0 );
    assert( !GL_ERROR_CHECK );

    return true;
}

///----------------------------------------------------------------------
///<summary>
///MRT̏I
///</summary>
///----------------------------------------------------------------------
bool DeferredRenderer::TerminateMRT()
{
    if ( mFrameBuffer )
    {
        glDeleteFramebuffersEXT( 1, &mFrameBuffer );
        mFrameBuffer = 0;
    }
    if ( mRenderBuffer )
    {
        glDeleteRenderbuffersEXT( 1, &mRenderBuffer );
        mRenderBuffer = 0;
    }

    for( uint32_t i=0; i< NUM_GBUFFER; ++i )
    {
        if ( mTexture[i] )
        {
        	glDeleteTextures( 1, &mTexture[i] );
            mTexture[i] = 0;
        }
    }

    assert( !GL_ERROR_CHECK );

    return true;
}

///----------------------------------------------------------------------
///<summary>
///
///</summary>
///----------------------------------------------------------------------
void DeferredRenderer::Release()
{
    mSaturn.Release();
    mGround.Release();
    TerminateMRT();
}

///----------------------------------------------------------------------
///<summary>
///_OȌ
///</summary>
///----------------------------------------------------------------------
void DeferredRenderer::PreRender()
{
    /* DO_NOTHING */
}

///----------------------------------------------------------------------
///<summary>
///ŝ̃_O
///</summary>
///----------------------------------------------------------------------
void DeferredRenderer::Render()
{
    //========================================
    // pass1. WIg
    //========================================

    // [hsvZ
    Matrix world;
    world.Identity();

    // r[svZ
    mCamera.Compute();

    // ˉesvZ
    Matrix proj = Math::CreatePerspectiveFieldOfView( Math::PiOver4, DemoApp::GetInstance()->GetAspectRatio(), 0.1f, 1000.0f );

    // p[^]
    cgSetMatrixParameterfr( mParam.projection, proj );
    cgSetMatrixParameterfr( mParam.view, mCamera.GetView() );
    cgSetParameter3fv( mParam.cameraPosition, mCamera.GetPosition() );

    glBindFramebufferEXT( GL_FRAMEBUFFER_EXT, mFrameBuffer );
    GLuint attachments[ 3 ] = 
    {
        GL_COLOR_ATTACHMENT0_EXT,
        GL_COLOR_ATTACHMENT1_EXT,
        GL_COLOR_ATTACHMENT2_EXT
    };
    glDrawBuffers( 3, attachments );
  
    // obt@NA
    glClearColor( 0.0f, 0.0f, 0.0f, 1.0f );
    glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

    // r[|[gݒ
    glViewport( 0, 0, (GLsizei)FRAME_BUFFER_WIDTH, (GLsizei)FRAME_BUFFER_HEIGHT );

    assert( !GL_ERROR_CHECK );
  
    // WIgpX
    {
       // pXXe[gݒ
        cgSetPassState( mpEffect->GetPass( 0, "Geometry" ) );

        // f`
        cgSetMatrixParameterfr( mParam.world, world );
        mSaturn.Draw();

        world = Math::CreateScale( 5.0f ) * Math::CreateTranslation( 0.0f, -1.0f, 0.0f );
        cgSetMatrixParameterfr( mParam.world, world );
        mGround.Draw();

        // pXXe[gZbg
        cgResetPassState( mpEffect->GetPass( 0, "Geometry" ) );
    }

    glBindFramebufferEXT( GL_FRAMEBUFFER_EXT, 0 );
    
    // Ă΂ȂĂ񂾂ǁCÔ
    glFlush();

    // Gobt@]
    cgGLSetTextureParameter( mParam.positionSmp, mTexture[ GBUFFER_1 ] );
    cgGLSetTextureParameter( mParam.normalDepthSmp, mTexture[ GBUFFER_2 ] );
    cgGLSetTextureParameter( mParam.diffuseAlbedoSmp, mTexture[ GBUFFER_3 ] );


    //========================================
    // pass2. CeBO
    //========================================

    // obt@NA
    glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

    // r[|[gݒ
    glViewport( 0, 0, DemoApp::GetInstance()->GetWidth(), DemoApp::GetInstance()->GetHeight() );
   
    // CeBOpX
    {
        cgSetPassState( mpEffect->GetPass( 0, "Lighting" ) );
 
        cgGLSetParameterArray4f( mParam.lightPostion, 0, NUM_LIGHT, &mLightPosition[0].x );
        cgGLSetParameterArray3f( mParam.lightColor, 0, NUM_LIGHT, &mLightColor[0].x );

        DrawRect();
 
        cgResetPassState( mpEffect->GetPass( 0, "Lighting" ) );
    }

    assert( !GL_ERROR_CHECK );
}

///----------------------------------------------------------------------
///<summary>
///``
///</summary>
///----------------------------------------------------------------------
void DeferredRenderer::DrawRect()
{
#if 1
    float aspect = (float)DemoApp::GetInstance()->GetWidth() / (float)DemoApp::GetInstance()->GetHeight();
 
    float vertex[] = {
        -1.0f * aspect,  1.0f * aspect, 0.0f,
        -1.0f * aspect, -1.0f * aspect, 0.0f,
         1.0f * aspect, -1.0f * aspect, 0.0f,
         1.0f * aspect,  1.0f * aspect, 0.0f 
    };
    float texcoord[] = {
        0.0f, 1.0f,
        0.0f, 0.0f,
        1.0f, 0.0f,
        1.0f, 1.0f 
    };

    glEnableClientState( GL_VERTEX_ARRAY );
    glVertexPointer( 3, GL_FLOAT, 0, vertex );

    glEnableClientState( GL_TEXTURE_COORD_ARRAY );
    glTexCoordPointer( 2, GL_FLOAT, 0, texcoord );

    glDrawArrays( GL_QUADS, 0, 4 );

    glDisableClientState( GL_TEXTURE_COORD_ARRAY );
    glDisableClientState( GL_VERTEX_ARRAY );
#else
    float aspect = (float)DemoApp::GetInstance()->GetWidth() / (float)DemoApp::GetInstance()->GetHeight();
    glBegin( GL_QUADS );
  
    glTexCoord2f( 0.0f, 1.0f );
    glVertex3f( -1.0f * aspect, 1.0f * aspect, 0.0f );

    glTexCoord2f( 0.0f, 0.0f );
    glVertex3f( -1.0f * aspect, -1.0f * aspect, 0.0f );

    glTexCoord2f( 1.0f, 0.0f );
    glVertex3f( 1.0f * aspect, -1.0f * aspect, 0.0f );

    glTexCoord2f( 1.0f, 1.0f );
    glVertex3f( 1.0f * aspect, 1.0f * aspect, 0.0f );

    glEnd();
#endif

}

///----------------------------------------------------------------------
///<summary>
///̂̃_O
///</summary>
///----------------------------------------------------------------------
void DeferredRenderer::RenderAlpha()
{
    /* DO_NOTHING */
}

///----------------------------------------------------------------------
///<summary>
///_Ȍ
///</summary>
///----------------------------------------------------------------------
void DeferredRenderer::PostRender()
{
    /* DO_NOTHING */
}

} // namespace SampleProgram
} // namespace Asura
} // namespace Schaft