//-----------------------------------------------------------------------
// File : DeferredRenderer.h
// Desc : Deferred Renderer
// Copyright(c) Project Asura All right reserved.
//-----------------------------------------------------------------------

//-----------------------------------------------------------------------
// History
//-----------------------------------------------------------------------
// <---> 2011/05/14 [Pocol] VK쐬.
// <End>

#ifndef __DEFERRED_RENDERER_H__
#define __DEFERRED_RENDERER_H__

//-----------------------------------------------------------------------
// Includes
//-----------------------------------------------------------------------
#include "AsDemoApp.h"
#include "AsDemoCamera.h"
#include "AsMath.h"
#include "AsShaderUtil.h"
#include "AsModelUtil.h"


namespace Schaft {
namespace Asura {
namespace SampleProgram {

/////////////////////////////////////////////////////////////////////////
// DeferedRenderer
/////////////////////////////////////////////////////////////////////////
class DeferredRenderer : public IRenderer
{
    //-------------------------------------------------------------------
    // list of friend classes and methods
    //-------------------------------------------------------------------
    /* NOTHING */

public:
    //-------------------------------------------------------------------
    // public variables
    //-------------------------------------------------------------------
    /* NOTHING */

    //-------------------------------------------------------------------
    // public methods
    //-------------------------------------------------------------------
    DeferredRenderer();
    virtual ~DeferredRenderer();
    bool Initialize();
    void PreRender();
    void Render();
    void RenderAlpha();
    void PostRender();
    void Release();

protected:
    //-------------------------------------------------------------------
    // protected variables
    //-------------------------------------------------------------------
    /* NOTHING */

    //-------------------------------------------------------------------
    // protected methods
    //-------------------------------------------------------------------
    void DrawRect();

private:
    //-------------------------------------------------------------------
    // private variables
    //-------------------------------------------------------------------
    struct EffectParam
    {
        CGparameter world;
        CGparameter view;
        CGparameter projection;
        CGparameter lightPostion;
        CGparameter lightColor;
        CGparameter cameraPosition;
        CGparameter positionSmp;
        CGparameter normalDepthSmp;
        CGparameter diffuseAlbedoSmp;
    };
    enum
    {
        GBUFFER_1 = 0,
        GBUFFER_2,
        GBUFFER_3,
        NUM_GBUFFER,
    };
    static const int NUM_LIGHT = 9;
    EffectParam                                 mParam;
    Schaft::Asura::Math::Vector4                mLightPosition[NUM_LIGHT];
    Schaft::Asura::Math::Vector3                mLightColor[NUM_LIGHT];
    GLuint                                      mTexture[NUM_GBUFFER];
    GLuint                                      mFrameBuffer;
    GLuint                                      mRenderBuffer;
    int                                         mPrevWidth;
    int                                         mPrevHeight;

    Schaft::Asura::Util::ModelData              mSaturn;
    Schaft::Asura::Util::ModelData              mGround;
    Schaft::Asura::SampleProgram::DemoCamera    mCamera;
    Schaft::Asura::Graphics::ICgEffect*         mpEffect;


    //-------------------------------------------------------------------
    // private methods
    //-------------------------------------------------------------------
    bool InitializeMRT( const int width, const int height );
    bool TerminateMRT();
};


} // namespace SampleProgram
} // namespace Asura
} // namespace Schaft

#endif//__DEFERRED_RENDERER_H__