//-----------------------------------------------------------------------
// File : FpsRenderer.cpp
// Desc : Frame Per Second Renderer
// Copyright(c) Project Asura All right reserved.
//-----------------------------------------------------------------------

//-----------------------------------------------------------------------
// History
//-----------------------------------------------------------------------
// <---> 2010/12/30 [Pocol] VK쐬.
// <End>

// Disable Warning C4996
#ifndef _CRT_SECURE_CPP_OVERLOAD_STANDARD_NAMES
#define _CRT_SECURE_CPP_OVERLOAD_STANDARD_NAMES 1
#endif
#ifndef _CRT_SECURE_CPP_OVERLOAD_SECURE_NAMES
#define _CRT_SECURE_CPP_OVERLOAD_SECURE_NAMES 1
#endif

//-----------------------------------------------------------------------
// Includes
//-----------------------------------------------------------------------
#include "FpsRenderer.h"
#include <GL/glew.h>
#include <GL/glut.h>

//-----------------------------------------------------------------------
// Using Statements
//-----------------------------------------------------------------------
using namespace Schaft::Asura;


namespace Schaft {
namespace Asura {
namespace SampleProgram {

/////////////////////////////////////////////////////////////////////////
// FpsRenderer 
/////////////////////////////////////////////////////////////////////////

///----------------------------------------------------------------------
///<summary>
///RXgN^
///</summary>
///----------------------------------------------------------------------
FpsRenderer::FpsRenderer() 
: mTimer()
{
	memset( mFpsText, 0, sizeof(unsigned char) * 50 );
	memset( mRendererText, 0, sizeof(unsigned char) * 50 );
	memset( mVendorText, 0, sizeof(unsigned char) * 50 );
	memset( mGLVerText, 0, sizeof(unsigned char) * 50 );

	mFrame = 0;
	mLastTime = 0;
}

///----------------------------------------------------------------------
///<summary>
///fXgN^
///</summary>
///----------------------------------------------------------------------
FpsRenderer::~FpsRenderer()
{
	/* DO_NOTHING */
}

///----------------------------------------------------------------------
///<summary>
///Еt
///</summary>
///----------------------------------------------------------------------
void FpsRenderer::Release()
{
	mTimer.Stop();
	mTimer.Reset();
}

///----------------------------------------------------------------------
///<summary>
///
///</summary>
///----------------------------------------------------------------------
bool FpsRenderer::Initialize()
{
	const GLubyte *version = glGetString( GL_VERSION );
	const GLubyte *vendor = glGetString( GL_VENDOR );
	const GLubyte *renderer = glGetString( GL_RENDERER );

	sprintf( mGLVerText, "OpenGL %s", version );
	sprintf( mVendorText, "%s", vendor );
	sprintf( mRendererText, "%s", renderer );

	mTimer.Start();

	return true;
}

///----------------------------------------------------------------------
///<summary>
///FPSXV
///</summary>
///----------------------------------------------------------------------
void FpsRenderer::PreRender()
{
	mFrame++;
	double time = mTimer.GetTime();
	if ( (time - mLastTime) >= 1.0 )
	{
		float fps = mFrame / (float)( time - mLastTime );
		sprintf( mFpsText, "%.3f FPS", fps );
		mFrame = 0;
		mLastTime = time;
	}
}

///----------------------------------------------------------------------
///<summary>
///`揈
///</summary>
///----------------------------------------------------------------------
void FpsRenderer::Render()
{
	bool isLighting = false;

	glMatrixMode( GL_PROJECTION );
	glPushMatrix();
	glLoadIdentity();
	gluOrtho2D( 0, DemoApp::GetInstance()->GetWidth(), DemoApp::GetInstance()->GetHeight(), 0 );
	glMatrixMode( GL_MODELVIEW );
	glPushMatrix();
	glLoadIdentity();

	if ( glIsEnabled( GL_LIGHTING ) )
	{
		isLighting = true;
		glDisable( GL_LIGHTING );
	}

	glColor4f( 1.0f, 1.0f, 1.0f, 1.0f );
	glRasterPos2i( 15, 15 );
	RenderText( mGLVerText );

	glRasterPos2i( 15, 30 );
	RenderText( mVendorText );

	glRasterPos2i( 15, 45 );
	RenderText( mRendererText );

	glRasterPos2i( 15, 60 );
	RenderText( mFpsText );

	if ( isLighting )
	{
		glEnable( GL_LIGHTING );
	}

	glPopMatrix();
	glLoadIdentity();
	glMatrixMode(GL_PROJECTION);
	glPopMatrix();
	glMatrixMode(GL_MODELVIEW);
}

///----------------------------------------------------------------------
///<summary>
///`̏
///</summary>
///----------------------------------------------------------------------
void FpsRenderer::PostRender()
{
	/* DO_NOTHING */
}

///----------------------------------------------------------------------
///<summary>
///At@l̂̕`
///</summary>
///----------------------------------------------------------------------
void FpsRenderer::RenderAlpha()
{
	/* DO_NOTHING */
}

///----------------------------------------------------------------------
///<summary>
///eLXg`
///</summary>
///<param name="text">`悷镶</param>
///----------------------------------------------------------------------
void FpsRenderer::RenderText( const char *text )
{
	for ( int i=0; i<(int)strlen(text); ++i )
	{
		glutBitmapCharacter( GLUT_BITMAP_HELVETICA_12, (int)text[i] );
	}
}

} // namespace SampleProgram
} // namespace Asura
} // namespace Schaft